@@ -41,14 +41,17 @@ type Server struct {
 }
 
 func NewGrpcServer(b *server.BgpServer, hosts string) *Server {
+	return NewServer(b, grpc.NewServer(), hosts)
+}
+
+func NewServer(b *server.BgpServer, g *grpc.Server, hosts string) *Server {
 	grpc.EnableTracing = false
-	grpcServer := grpc.NewServer()
 	server := &Server{
 		bgpServer:  b,
-		grpcServer: grpcServer,
+		grpcServer: g,
 		hosts:      hosts,
 	}
-	RegisterGobgpApiServer(grpcServer, server)
+	RegisterGobgpApiServer(g, server)
 	return server
 }
 
@@ -718,6 +721,7 @@ func (s *Server) GetVrf(ctx context.Context, arg *GetVrfRequest) (*GetVrfRespons
 		return &Vrf{
 			Name:     v.Name,
 			Rd:       rd,
+			Id:       v.Id,
 			ImportRt: f(v.ImportRt),
 			ExportRt: f(v.ExportRt),
 		}
@@ -901,6 +905,11 @@ func NewDefinedSetFromApiStruct(a *DefinedSet) (table.DefinedSet, error) {
 			ExtCommunitySetName: a.Name,
 			ExtCommunityList:    a.List,
 		})
+	case table.DEFINED_TYPE_LARGE_COMMUNITY:
+		return table.NewLargeCommunitySet(config.LargeCommunitySet{
+			LargeCommunitySetName: a.Name,
+			LargeCommunityList:    a.List,
+		})
 	default:
 		return nil, fmt.Errorf("invalid defined type")
 	}
@@ -956,6 +965,14 @@ func (s *Server) GetDefinedSet(ctx context.Context, arg *GetDefinedSetRequest) (
 		}
 		sets = append(sets, ad)
 	}
+	for _, cs := range cd.BgpDefinedSets.LargeCommunitySets {
+		ad := &DefinedSet{
+			Type: DefinedType_LARGE_COMMUNITY,
+			Name: cs.LargeCommunitySetName,
+			List: cs.LargeCommunityList,
+		}
+		sets = append(sets, ad)
+	}
 	for _, cs := range cd.BgpDefinedSets.AsPathSets {
 		ad := &DefinedSet{
 			Type: DefinedType_AS_PATH,
@@ -1030,6 +1047,12 @@ func toStatementApi(s *config.Statement) *Statement {
 			Name: s.Conditions.BgpConditions.MatchExtCommunitySet.ExtCommunitySet,
 		}
 	}
+	if s.Conditions.BgpConditions.MatchLargeCommunitySet.LargeCommunitySet != "" {
+		cs.CommunitySet = &MatchSet{
+			Type: MatchType(s.Conditions.BgpConditions.MatchLargeCommunitySet.MatchSetOptions.ToInt()),
+			Name: s.Conditions.BgpConditions.MatchLargeCommunitySet.LargeCommunitySet,
+		}
+	}
 	if s.Conditions.BgpConditions.RouteType != "" {
 		cs.RouteType = Conditions_RouteType(s.Conditions.BgpConditions.RouteType.ToInt())
 	}
@@ -1092,6 +1115,15 @@ func toStatementApi(s *config.Statement) *Statement {
 				Communities: s.Actions.BgpActions.SetExtCommunity.SetExtCommunityMethod.CommunitiesList,
 			}
 		}(),
+		LargeCommunity: func() *CommunityAction {
+			if len(s.Actions.BgpActions.SetLargeCommunity.SetLargeCommunityMethod.CommunitiesList) == 0 {
+				return nil
+			}
+			return &CommunityAction{
+				Type:        CommunityActionType(config.BgpSetCommunityOptionTypeToIntMap[config.BgpSetCommunityOptionType(s.Actions.BgpActions.SetLargeCommunity.Options)]),
+				Communities: s.Actions.BgpActions.SetLargeCommunity.SetLargeCommunityMethod.CommunitiesList,
+			}
+		}(),
 		Nexthop: func() *NexthopAction {
 			if len(string(s.Actions.BgpActions.SetNextHop)) == 0 {
 				return nil
@@ -1251,6 +1283,21 @@ func NewExtCommunityConditionFromApiStruct(a *MatchSet) (*table.ExtCommunityCond
 	return table.NewExtCommunityCondition(c)
 }
 
+func NewLargeCommunityConditionFromApiStruct(a *MatchSet) (*table.LargeCommunityCondition, error) {
+	if a == nil {
+		return nil, nil
+	}
+	typ, err := toConfigMatchSetOption(a.Type)
+	if err != nil {
+		return nil, err
+	}
+	c := config.MatchLargeCommunitySet{
+		LargeCommunitySet: a.Name,
+		MatchSetOptions:   typ,
+	}
+	return table.NewLargeCommunityCondition(c)
+}
+
 func NewRoutingActionFromApiStruct(a RouteAction) (*table.RoutingAction, error) {
 	if a == RouteAction_NONE {
 		return nil, nil
@@ -1288,11 +1335,23 @@ func NewExtCommunityActionFromApiStruct(a *CommunityAction) (*table.ExtCommunity
 	})
 }
 
+func NewLargeCommunityActionFromApiStruct(a *CommunityAction) (*table.LargeCommunityAction, error) {
+	if a == nil {
+		return nil, nil
+	}
+	return table.NewLargeCommunityAction(config.SetLargeCommunity{
+		Options: config.IntToBgpSetCommunityOptionTypeMap[int(a.Type)],
+		SetLargeCommunityMethod: config.SetLargeCommunityMethod{
+			CommunitiesList: a.Communities,
+		},
+	})
+}
+
 func NewMedActionFromApiStruct(a *MedAction) (*table.MedAction, error) {
 	if a == nil {
 		return nil, nil
 	}
-	return table.NewMedActionFromApiStruct(table.MedActionType(a.Type), int(a.Value)), nil
+	return table.NewMedActionFromApiStruct(table.MedActionType(a.Type), a.Value), nil
 }
 
 func NewLocalPrefActionFromApiStruct(a *LocalPrefAction) (*table.LocalPrefAction, error) {
@@ -1365,6 +1424,9 @@ func NewStatementFromApiStruct(a *Statement) (*table.Statement, error) {
 			func() (table.Condition, error) {
 				return NewExtCommunityConditionFromApiStruct(a.Conditions.ExtCommunitySet)
 			},
+			func() (table.Condition, error) {
+				return NewLargeCommunityConditionFromApiStruct(a.Conditions.LargeCommunitySet)
+			},
 		}
 		cs = make([]table.Condition, 0, len(cfs))
 		for _, f := range cfs {
@@ -1389,6 +1451,9 @@ func NewStatementFromApiStruct(a *Statement) (*table.Statement, error) {
 			func() (table.Action, error) {
 				return NewExtCommunityActionFromApiStruct(a.Actions.ExtCommunity)
 			},
+			func() (table.Action, error) {
+				return NewLargeCommunityActionFromApiStruct(a.Actions.LargeCommunity)
+			},
 			func() (table.Action, error) {
 				return NewMedActionFromApiStruct(a.Actions.Med)
 			},
@@ -1630,8 +1695,6 @@ func (s *Server) GetServer(ctx context.Context, arg *GetServerRequest) (*GetServ
 			RouterId:         g.Config.RouterId,
 			ListenPort:       g.Config.Port,
 			ListenAddresses:  g.Config.LocalAddressList,
-			MplsLabelMin:     g.MplsLabelRange.MinLabel,
-			MplsLabelMax:     g.MplsLabelRange.MaxLabel,
 			UseMultiplePaths: g.UseMultiplePaths.Config.Enabled,
 		},
 	}, nil
@@ -1663,10 +1726,6 @@ func (s *Server) StartServer(ctx context.Context, arg *StartServerRequest) (*Sta
 				Port:             g.ListenPort,
 				LocalAddressList: g.ListenAddresses,
 			},
-			MplsLabelRange: config.MplsLabelRange{
-				MinLabel: g.MplsLabelMin,
-				MaxLabel: g.MplsLabelMax,
-			},
 			AfiSafis: families,
 			UseMultiplePaths: config.UseMultiplePaths{
 				Config: config.UseMultiplePathsConfig{
@@ -1681,3 +1740,37 @@ func (s *Server) StartServer(ctx context.Context, arg *StartServerRequest) (*Sta
 func (s *Server) StopServer(ctx context.Context, arg *StopServerRequest) (*StopServerResponse, error) {
 	return &StopServerResponse{}, s.bgpServer.Stop()
 }
+
+func (s *Server) GetRibInfo(ctx context.Context, arg *GetRibInfoRequest) (*GetRibInfoResponse, error) {
+	family := bgp.RouteFamily(arg.Info.Family)
+	var in bool
+	var err error
+	var info *table.TableInfo
+	switch arg.Info.Type {
+	case Resource_GLOBAL, Resource_LOCAL:
+		info, err = s.bgpServer.GetRibInfo(arg.Info.Name, family)
+	case Resource_ADJ_IN:
+		in = true
+		fallthrough
+	case Resource_ADJ_OUT:
+		info, err = s.bgpServer.GetAdjRibInfo(arg.Info.Name, family, in)
+	default:
+		return nil, fmt.Errorf("unsupported resource type: %s", arg.Info.Type)
+	}
+
+	if err != nil {
+		return nil, err
+	}
+
+	return &GetRibInfoResponse{
+		Info: &TableInfo{
+			Type:           arg.Info.Type,
+			Family:         arg.Info.Family,
+			Name:           arg.Info.Name,
+			NumDestination: uint64(info.NumDestination),
+			NumPath:        uint64(info.NumPath),
+			NumAccepted:    uint64(info.NumAccepted),
+		},
+	}, nil
+
+}
