@@ -217,6 +217,8 @@ func (server *BgpServer) Serve() {
 		}
 
 		select {
+		case f := <-server.mgmtCh:
+			f()
 		case conn := <-server.acceptCh:
 			passConn(conn)
 		default:
@@ -360,23 +362,25 @@ func filterpath(peer *Peer, path *table.Path, withdrawals []*table.Path) *table.
 	}
 
 	if peer.ID() == path.GetSource().Address.String() {
-		// Say, gobgp was advertising prefix A and peer P also.
-		// When gobgp withdraws prefix A, best path calculation chooses
-		// the path from P as the best path for prefix A.
-		// For peers other than P, this path should be advertised
-		// (as implicit withdrawal). However for P, we should advertise
-		// the local withdraw path.
-
 		// Note: multiple paths having the same prefix could exist the
 		// withdrawals list in the case of Route Server setup with
 		// import policies modifying paths. In such case, gobgp sends
 		// duplicated update messages; withdraw messages for the same
 		// prefix.
-		// However, currently we don't support local path for Route
-		// Server setup so this is NOT the case.
-		for _, w := range withdrawals {
-			if w.IsLocal() && path.GetNlri().String() == w.GetNlri().String() {
-				return w
+		if !peer.isRouteServerClient() {
+			// Say, peer A and B advertized same prefix P, and
+			// best path calculation chose a path from B as best.
+			// When B withdraws prefix P, best path calculation chooses
+			// the path from A as best.
+			// For peers other than A, this path should be advertised
+			// (as implicit withdrawal). However for A, we should advertise
+			// the withdrawal path.
+			// Thing is same when peer A and we advertized prefix P (as local
+			// route), then, we withdraws the prefix.
+			for _, w := range withdrawals {
+				if path.GetNlri().String() == w.GetNlri().String() {
+					return w
+				}
 			}
 		}
 		log.WithFields(log.Fields{
@@ -413,7 +417,7 @@ func (server *BgpServer) notifyBestWatcher(best map[string][]*table.Path, multip
 		switch p.GetRouteFamily() {
 		case bgp.RF_IPv4_VPN, bgp.RF_IPv6_VPN:
 			for _, vrf := range server.globalRib.Vrfs {
-				if table.CanImportToVrf(vrf, p) {
+				if vrf.Id != 0 && table.CanImportToVrf(vrf, p) {
 					p.VrfIds = append(p.VrfIds, uint16(vrf.Id))
 				}
 			}
@@ -626,13 +630,72 @@ func (server *BgpServer) handleFSMMessage(peer *Peer, e *FsmMsg) {
 			peer.DropAll(drop)
 			server.dropPeerAllRoutes(peer, drop)
 		} else if peer.fsm.pConf.GracefulRestart.State.PeerRestarting && nextState == bgp.BGP_FSM_IDLE {
-			// RFC 4724 4.2
-			// If the session does not get re-established within the "Restart Time"
-			// that the peer advertised previously, the Receiving Speaker MUST
-			// delete all the stale routes from the peer that it is retaining.
-			peer.fsm.pConf.GracefulRestart.State.PeerRestarting = false
-			peer.DropAll(peer.configuredRFlist())
-			server.dropPeerAllRoutes(peer, peer.configuredRFlist())
+			if peer.fsm.pConf.GracefulRestart.State.LongLivedEnabled {
+				llgr, no_llgr := peer.llgrFamilies()
+
+				peer.DropAll(no_llgr)
+				server.dropPeerAllRoutes(peer, no_llgr)
+
+				// attach LLGR_STALE community to paths in peer's adj-rib-in
+				// paths with NO_LLGR are deleted
+				pathList := peer.markLLGRStale(llgr)
+
+				// calculate again
+				// wheh path with LLGR_STALE chosen as best,
+				// peer which doesn't support LLGR will drop the path
+				// if it is in adj-rib-out, do withdrawal
+				server.propagateUpdate(peer, pathList)
+
+				for _, f := range llgr {
+					endCh := make(chan struct{})
+					peer.llgrEndChs = append(peer.llgrEndChs, endCh)
+					go func(family bgp.RouteFamily, endCh chan struct{}) {
+						t := peer.llgrRestartTime(family)
+						timer := time.NewTimer(time.Second * time.Duration(t))
+
+						log.WithFields(log.Fields{
+							"Topic":  "Peer",
+							"Key":    peer.ID(),
+							"Family": family,
+						}).Debugf("start LLGR restart timer (%d sec) for %s", t, family)
+
+						select {
+						case <-timer.C:
+							ch := make(chan struct{})
+							defer func() { <-ch }()
+							server.mgmtCh <- func() {
+								log.WithFields(log.Fields{
+									"Topic":  "Peer",
+									"Key":    peer.ID(),
+									"Family": family,
+								}).Debugf("LLGR restart timer (%d sec) for %s expired", t, family)
+								defer close(ch)
+								peer.DropAll([]bgp.RouteFamily{family})
+								server.dropPeerAllRoutes(peer, []bgp.RouteFamily{family})
+
+								// when all llgr restart timer expired, stop PeerRestarting
+								if peer.llgrRestartTimerExpired(family) {
+									peer.stopPeerRestarting()
+								}
+							}
+						case <-endCh:
+							log.WithFields(log.Fields{
+								"Topic":  "Peer",
+								"Key":    peer.ID(),
+								"Family": family,
+							}).Debugf("stop LLGR restart timer (%d sec) for %s", t, family)
+						}
+					}(f, endCh)
+				}
+			} else {
+				// RFC 4724 4.2
+				// If the session does not get re-established within the "Restart Time"
+				// that the peer advertised previously, the Receiving Speaker MUST
+				// delete all the stale routes from the peer that it is retaining.
+				peer.fsm.pConf.GracefulRestart.State.PeerRestarting = false
+				peer.DropAll(peer.configuredRFlist())
+				server.dropPeerAllRoutes(peer, peer.configuredRFlist())
+			}
 		}
 
 		peer.outgoing.Close()
@@ -649,8 +712,7 @@ func (server *BgpServer) handleFSMMessage(peer *Peer, e *FsmMsg) {
 
 					server.mgmtCh <- func() {
 						defer close(ch)
-
-						server.softResetOut(peer.fsm.pConf.Config.NeighborAddress, bgp.RouteFamily(0), true)
+						server.softResetOut(peer.fsm.pConf.Config.NeighborAddress, family, true)
 					}
 				}
 			}
@@ -830,7 +892,7 @@ func (server *BgpServer) handleFSMMessage(peer *Peer, e *FsmMsg) {
 				}
 				if peer.fsm.pConf.GracefulRestart.State.PeerRestarting {
 					if peer.recvedAllEOR() {
-						peer.fsm.pConf.GracefulRestart.State.PeerRestarting = false
+						peer.stopPeerRestarting()
 						pathList := peer.adjRibIn.DropStale(peer.configuredRFlist())
 						log.WithFields(log.Fields{
 							"Topic": "Peer",
@@ -1047,12 +1109,8 @@ func (server *BgpServer) fixupApiPath(vrfId string, pathList []*table.Path) erro
 		}
 
 		if vrfId != "" {
-			label, err := server.globalRib.GetNextLabel(vrfId, path.GetNexthop().String(), path.IsWithdraw)
-			if err != nil {
-				return err
-			}
 			vrf := server.globalRib.Vrfs[vrfId]
-			if err := vrf.ToGlobalPath(path, label); err != nil {
+			if err := vrf.ToGlobalPath(path); err != nil {
 				return err
 			}
 		}
@@ -1169,7 +1227,7 @@ func (s *BgpServer) Start(c *config.Global) (err error) {
 		}
 
 		rfs, _ := config.AfiSafis(c.AfiSafis).ToRfList()
-		s.globalRib = table.NewTableManager(rfs, c.MplsLabelRange.MinLabel, c.MplsLabelRange.MaxLabel)
+		s.globalRib = table.NewTableManager(rfs)
 
 		if err = s.policy.Reset(&config.RoutingPolicy{}, map[string]config.ApplyPolicy{}); err != nil {
 			return
@@ -1497,6 +1555,56 @@ func (s *BgpServer) GetAdjRib(addr string, family bgp.RouteFamily, in bool, pref
 	return
 }
 
+func (s *BgpServer) GetRibInfo(addr string, family bgp.RouteFamily) (info *table.TableInfo, err error) {
+	ch := make(chan struct{})
+	defer func() { <-ch }()
+
+	s.mgmtCh <- func() {
+		defer close(ch)
+
+		m := s.globalRib
+		id := table.GLOBAL_RIB_NAME
+		if len(addr) > 0 {
+			peer, ok := s.neighborMap[addr]
+			if !ok {
+				err = fmt.Errorf("Neighbor that has %v doesn't exist.", addr)
+				return
+			}
+			if !peer.isRouteServerClient() {
+				err = fmt.Errorf("Neighbor %v doesn't have local rib", addr)
+				return
+			}
+			id = peer.ID()
+		}
+		info, err = m.TableInfo(id, family)
+	}
+	return
+}
+
+func (s *BgpServer) GetAdjRibInfo(addr string, family bgp.RouteFamily, in bool) (info *table.TableInfo, err error) {
+	ch := make(chan struct{})
+	defer func() { <-ch }()
+
+	s.mgmtCh <- func() {
+		defer close(ch)
+
+		peer, ok := s.neighborMap[addr]
+		if !ok {
+			err = fmt.Errorf("Neighbor that has %v doesn't exist.", addr)
+			return
+		}
+
+		var adjRib *table.AdjRib
+		if in {
+			adjRib = peer.adjRibIn
+		} else {
+			adjRib = peer.adjRibOut
+		}
+		info, err = adjRib.TableInfo(family)
+	}
+	return
+}
+
 func (s *BgpServer) GetServer() (c *config.Global) {
 	ch := make(chan struct{})
 	defer func() { <-ch }()
@@ -1611,6 +1719,7 @@ func (server *BgpServer) deleteNeighbor(c *config.Neighbor, code, subcode uint8)
 	}).Infof("Delete a peer configuration for:%s", addr)
 
 	n.fsm.sendNotification(code, subcode, nil, "")
+	n.stopPeerRestarting()
 
 	go func(addr string) {
 		t1 := time.AfterFunc(time.Minute*5, func() {
